@@ -1,10 +1,10 @@
 package Method::Generate::DemolishAll;
 
-use Moo::_strictures;
-use Moo::Object ();
-our @ISA = qw(Moo::Object);
-use Sub::Quote qw(quote_sub quotify);
+use strictures 1;
+use base qw(Moo::Object);
+use Sub::Quote;
 use Moo::_Utils;
+use B qw(perlstring);
 
 sub generate_method {
   my ($self, $into) = @_;
@@ -18,16 +18,14 @@ sub generate_method {
     my $e = do {
       local $?;
       local $@;
-      require Devel::GlobalDestruction;
+      require Moo::_Utils;
       eval {
-        $self->DEMOLISHALL(Devel::GlobalDestruction::in_global_destruction);
+        $self->DEMOLISHALL(Moo::_Utils::_in_global_destruction);
       };
       $@;
     };
-
-    # fatal warnings+die in DESTROY = bad times (perl rt#123398)
-    no warnings FATAL => 'all';
-    use warnings 'all';
+  
+    no warnings 'misc';
     die $e if $e; # rethrow
   !;
 }
@@ -43,7 +41,7 @@ sub demolishall_body_for {
 
 sub _handle_subdemolish {
   my ($self, $into) = @_;
-  '    if (ref($_[0]) ne '.quotify($into).') {'."\n".
+  '    if (ref($_[0]) ne '.perlstring($into).') {'."\n".
   '      return shift->Moo::Object::DEMOLISHALL(@_)'.";\n".
   '    }'."\n";
 }
