@@ -4,8 +4,9 @@ use strictures 1;
 use Moo::_Utils;
 use B 'perlstring';
 use Sub::Defer ();
+use Import::Into;
 
-our $VERSION = '1.003001';
+our $VERSION = '1.004000';
 $VERSION = eval $VERSION;
 
 require Moo::sification;
@@ -21,7 +22,8 @@ sub _install_tracked {
 sub import {
   my $target = caller;
   my $class = shift;
-  strictures->import;
+  _set_loaded(caller);
+  strictures->import::into(1);
   if ($Role::Tiny::INFO{$target} and $Role::Tiny::INFO{$target}{is_role}) {
     die "Cannot import Moo into a role";
   }
@@ -65,6 +67,9 @@ sub import {
     };
   }
   return if $MAKERS{$target}{is_class}; # already exported into this package
+  my $stash = _getstash($target);
+  my @not_methods = map { *$_{CODE}||() } grep !ref($_), values %$stash;
+  @{$MAKERS{$target}{not_methods}={}}{@not_methods} = @not_methods;
   $MAKERS{$target}{is_class} = 1;
   {
     no strict 'refs';
@@ -121,7 +126,7 @@ sub _accessor_maker_for {
   $MAKERS{$target}{accessor} ||= do {
     my $maker_class = do {
       if (my $m = do {
-            if (my $defer_target = 
+            if (my $defer_target =
                   (Sub::Defer::defer_info($target->can('new'))||[])->[0]
               ) {
               my ($pkg) = ($defer_target =~ /^(.*)::[^:]+$/);
@@ -166,15 +171,20 @@ sub _constructor_maker_for {
       } else {
         $moo_constructor = 1; # no other constructor, make a Moo one
       }
-    };
+    }
     ($con ? ref($con) : 'Method::Generate::Constructor')
       ->new(
         package => $target,
         accessor_generator => $class->_accessor_maker_for($target),
-        construction_string => (
-          $moo_constructor
-            ? ($con ? $con->construction_string : undef)
-            : ('$class->'.$target.'::SUPER::new($class->can(q[FOREIGNBUILDARGS]) ? $class->FOREIGNBUILDARGS(@_) : @_)')
+        $moo_constructor ? (
+          $con ? (construction_string => $con->construction_string) : ()
+        ) : (
+          construction_builder => sub {
+            '$class->'.$target.'::SUPER::new('
+              .($target->can('FOREIGNBUILDARGS') ?
+                '$class->FOREIGNBUILDARGS(@_)' : '@_')
+              .')'
+          },
         ),
         subconstructor_handler => (
           '      if ($Moo::MAKERS{$class}) {'."\n"
@@ -190,7 +200,26 @@ sub _constructor_maker_for {
   }
 }
 
+sub _concrete_methods_of {
+  my ($me, $role) = @_;
+  my $makers = $MAKERS{$role};
+  # grab role symbol table
+  my $stash = _getstash($role);
+  # reverse so our keys become the values (captured coderefs) in case
+  # they got copied or re-used since
+  my $not_methods = { reverse %{$makers->{not_methods}||{}} };
+  +{
+    # grab all code entries that aren't in the not_methods list
+    map {
+      my $code = *{$stash->{$_}}{CODE};
+      ( ! $code or exists $not_methods->{$code} ) ? () : ($_ => $code)
+    } grep !ref($stash->{$_}), keys %$stash
+  };
+}
+
 1;
+__END__
+
 =pod
 
 =encoding utf-8
@@ -204,6 +233,7 @@ Moo - Minimalist Object Orientation (with Moose compatibility)
  package Cat::Food;
 
  use Moo;
+ use namespace::clean;
 
  sub feed_lion {
    my $self = shift;
@@ -420,7 +450,9 @@ Returns true if the object composes in the passed role.
  extends 'Parent::Class';
 
 Declares base class. Multiple superclasses can be passed for multiple
-inheritance (but please use roles instead).
+inheritance (but please use roles instead).  The class will be loaded, however
+no errors will be triggered if it can't be found and there are already subs in
+the class.
 
 Calling extends more than once will REPLACE your superclasses, not add to
 them like 'use base' would.
@@ -434,7 +466,8 @@ or
  with 'Some::Role1', 'Some::Role2';
 
 Composes one or more L<Moo::Role> (or L<Role::Tiny>) roles into the current
-class.  An error will be raised if these roles have conflicting methods.
+class.  An error will be raised if these roles have conflicting methods.  The
+roles will be loaded using the same mechansim as C<extends> uses.
 
 =head2 has
 
@@ -701,9 +734,11 @@ aware can take advantage of this.
 
 To do this, you can write
 
-  use Moo;
   use Sub::Quote;
 
+  use Moo;
+  use namespace::clean;
+
   has foo => (
     is => 'ro',
     isa => quote_sub(q{ die "Not <3" unless $_[0] < 3 })
@@ -733,6 +768,48 @@ which will be inlined as
 See L<Sub::Quote> for more information, including how to pass lexical
 captures that will also be compiled into the subroutine.
 
+=head1 CLEANING UP IMPORTS
+
+L<Moo> will not clean up imported subroutines for you; you will have
+to do that manually. The recommended way to do this is to declare your
+imports first, then C<use Moo>, then C<use namespace::clean>.
+Anything imported before L<namespace::clean> will be scrubbed.
+Anything imported or declared after will be still be available.
+
+ package Record;
+
+ use Digest::MD5 qw(md5_hex);
+
+ use Moo;
+ use namespace::clean;
+
+ has name => (is => 'ro', required => 1);
+ has id => (is => 'lazy');
+ sub _build_id {
+   my ($self) = @_;
+   return md5_hex($self->name);
+ }
+
+ 1;
+
+If you were to import C<md5_hex> after L<namespace::clean> you would
+be able to call C<< ->md5_hex() >> on your C<Record> instances (and it
+probably wouldn't do what you expect!).
+
+L<Moo::Role>s behave slightly differently.  Since their methods are
+composed into the consuming class, they can do a little more for you
+automatically.  As long as you declare your imports before calling
+C<use Moo::Role>, those imports and the ones L<Moo::Role> itself
+provides will not be composed into consuming classes, so there's usually
+no need to use L<namespace::clean>.
+
+B<On L<namespace::autoclean>:> If you're coming to Moo from the Moose
+world, you may be accustomed to using L<namespace::autoclean> in all
+your packages. This is not recommended for L<Moo> packages, because
+L<namespace::autoclean> will inflate your class to a full L<Moose>
+class.  It'll work, but you will lose the benefits of L<Moo>.  Instead
+you are recommended to just use L<namespace::clean>.
+
 =head1 INCOMPATIBILITIES WITH MOOSE
 
 There is no built-in type system.  C<isa> is verified with a coderef; if you
@@ -796,8 +873,16 @@ Since C<coerce> does not require C<isa> to be defined but L<Moose> does
 require it, the metaclass inflation for coerce alone is a trifle insane
 and if you attempt to subtype the result will almost certainly break.
 
-Handling of warnings: when you C<use Moo> we enable FATAL warnings.  The nearest
-similar invocation for L<Moose> would be:
+C<BUILDARGS> is not triggered if your class does not have any attributes.
+Without attributes, C<BUILDARGS> return value would be ignored, so we just
+skip calling the method instead.
+
+Handling of warnings: when you C<use Moo> we enable FATAL warnings, and some
+several extra pragmas when used in development: L<indirect>,
+L<multidimensional>, and L<bareword::filehandles>.  See the L<strictures>
+documentation for the details on this.
+
+A similar invocation for L<Moose> would be:
 
   use Moose;
   use warnings FATAL => "all";
