@@ -4,8 +4,9 @@ use strictures 1;
 use Moo::_Utils;
 use Role::Tiny ();
 use base qw(Role::Tiny);
+use Import::Into;
 
-our $VERSION = '1.003001';
+our $VERSION = '1.004000';
 $VERSION = eval $VERSION;
 
 require Moo::sification;
@@ -24,7 +25,8 @@ sub _install_tracked {
 sub import {
   my $target = caller;
   my ($me) = @_;
-  strictures->import;
+  _set_loaded(caller);
+  strictures->import::into(1);
   if ($Moo::MAKERS{$target} and $Moo::MAKERS{$target}{is_class}) {
     die "Cannot import Moo::Role into a Moo class";
   }
@@ -102,6 +104,13 @@ sub _maybe_reset_handlemoose {
   }
 }
 
+sub methods_provided_by {
+  my ($self, $role) = @_;
+  $self->_inhale_if_moose($role);
+  die "${role} is not a Moo::Role" unless $INFO{$role};
+  return $self->SUPER::methods_provided_by($role);
+}
+
 sub _inhale_if_moose {
   my ($self, $role) = @_;
   _load_module($role);
@@ -118,8 +127,11 @@ sub _inhale_if_moose {
         and $meta->isa('Mouse::Meta::Role')
      )
   ) {
+    my $is_mouse = $meta->isa('Mouse::Meta::Role');
     $INFO{$role}{methods} = {
       map +($_ => $role->can($_)),
+        grep $role->can($_),
+        grep !($is_mouse && $_ eq 'meta'),
         grep !$meta->get_method($_)->isa('Class::MOP::Method::Meta'),
           $meta->get_method_list
     };
@@ -130,7 +142,6 @@ sub _inhale_if_moose {
     $INFO{$role}{attributes} = [
       map +($_ => do {
         my $attr = $meta->get_attribute($_);
-        my $is_mouse = $meta->isa('Mouse::Meta::Role');
         my $spec = { %{ $is_mouse ? $attr : $attr->original_options } };
 
         if ($spec->{isa}) {
@@ -255,6 +266,7 @@ sub create_class_with_roles {
     # old fashioned way time.
     *{_getglob("${new_name}::ISA")} = [ $superclass ];
     $me->apply_roles_to_package($new_name, @roles);
+    _set_loaded($new_name, (caller)[1]);
     return $new_name;
   }
 
@@ -270,12 +282,14 @@ sub create_class_with_roles {
 
   $me->_handle_constructor($new_name, $_) for @roles;
 
+  _set_loaded($new_name, (caller)[1]);
   return $new_name;
 }
 
 sub apply_roles_to_object {
   my ($me, $object, @roles) = @_;
   my $new = $me->SUPER::apply_roles_to_object($object, @roles);
+  _set_loaded(ref $new, (caller)[1]);
 
   my $apply_defaults = $APPLY_DEFAULTS{ref $new} ||= do {
     my %attrs = map { @{$INFO{$_}{attributes}||[]} } @roles;
@@ -343,6 +357,7 @@ sub _handle_constructor {
 }
 
 1;
+__END__
 
 =head1 NAME
 
@@ -397,6 +412,33 @@ imported by this module.
 Declares an attribute for the class to be composed into.  See
 L<Moo/has> for all options.
 
+=head1 CLEANING UP IMPORTS
+
+L<Moo::Role> cleans up its own imported methods and any imports
+declared before the C<use Moo::Role> statement automatically.
+Anything imported after C<use Moo::Role> will be composed into
+consuming packages.  A package that consumes this role:
+
+ package My::Role::ID;
+
+ use Digest::MD5 qw(md5_hex);
+ use Moo::Role;
+ use Digest::SHA qw(sha1_hex);
+
+ requires 'name';
+
+ sub as_md5  { my ($self) = @_; return md5_hex($self->name);  }
+ sub as_sha1 { my ($self) = @_; return sha1_hex($self->name); }
+
+ 1;
+
+..will now have a C<< $self->sha1_hex() >> method available to it
+that probably does not do what you expect.  On the other hand, a call
+to C<< $self->md5_hex() >> will die with the helpful error message:
+C<Can't locate object method "md5_hex">.
+
+See L<Moo/"CLEANING UP IMPORTS"> for more details.
+
 =head1 SUPPORT
 
 See L<Moo> for support and contact information.
@@ -408,3 +450,5 @@ See L<Moo> for authors.
 =head1 COPYRIGHT AND LICENSE
 
 See L<Moo> for the copyright and license.
+
+=cut
