@@ -9,15 +9,17 @@ use constant lt_5_8_3 => ( $] < 5.008003 or $ENV{MOO_TEST_PRE_583} ) ? 1 : 0;
 use constant can_haz_subname => eval { require Sub::Name };
 
 use strictures 1;
-use Module::Runtime qw(require_module);
+use Module::Runtime qw(use_package_optimistically module_notional_filename);
+
 use Devel::GlobalDestruction ();
 use base qw(Exporter);
 use Moo::_mro;
+use Config;
 
 our @EXPORT = qw(
     _getglob _install_modifier _load_module _maybe_load_module
     _get_linear_isa _getstash _install_coderef _name_coderef
-    _unimport_coderefs _in_global_destruction
+    _unimport_coderefs _in_global_destruction _set_loaded
 );
 
 sub _in_global_destruction ();
@@ -37,14 +39,21 @@ sub _install_modifier {
 our %MAYBE_LOADED;
 
 sub _load_module {
-  (my $proto = $_[0]) =~ s/::/\//g;
-  return 1 if $INC{"${proto}.pm"};
+  my $module = $_[0];
+  my $file = module_notional_filename($module);
+  use_package_optimistically($module);
+  return 1
+    if $INC{$file};
+  my $error = $@ || "Can't locate $file";
+
   # can't just ->can('can') because a sub-package Foo::Bar::Baz
   # creates a 'Baz::' key in Foo::Bar's symbol table
-  my $stash = _getstash($_[0])||{};
+  my $stash = _getstash($module)||{};
   return 1 if grep +(!ref($_) and *$_{CODE}), values %$stash;
-  require_module($_[0]);
-  return 1;
+  return 1
+    if $INC{"Moose.pm"} && Class::MOP::class_of($module)
+    or Mouse::Util->can('find_meta') && Mouse::Util::find_meta($module);
+  die $error;
 }
 
 sub _maybe_load_module {
@@ -62,6 +71,10 @@ sub _maybe_load_module {
   return $MAYBE_LOADED{$_[0]};
 }
 
+sub _set_loaded {
+  $INC{Module::Runtime::module_notional_filename($_[0])} ||= $_[1];
+}
+
 sub _get_linear_isa {
   return mro::get_linear_isa($_[0]);
 }
@@ -97,20 +110,8 @@ sub _unimport_coderefs {
   }
 }
 
-sub STANDARD_DESTROY {
-  my $self = shift;
-
-  my $e = do {
-    local $?;
-    local $@;
-    eval {
-      $self->DEMOLISHALL(_in_global_destruction);
-    };
-    $@;
-  };
-
-  no warnings 'misc';
-  die $e if $e; # rethrow
+if ($Config{useithreads}) {
+  require Moo::HandleMoose::_TypeMap;
 }
 
 1;
